import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

import os

os.makedirs('./output/', exist_ok=True)

data = pd.read_csv('./data/figure2/figure2.csv')


# figure 2a

import numpy as np
from sklearn.metrics import r2_score

r2 = r2_score(data['mrggpu threshold'], data['threshold (mA)'])

fig = plt.figure(dpi=200, figsize=(3, 3))
imthera_data = data[(data['cuff']=='ImThera')]
livanova_data = data[(data['cuff']=='Helical')]

imth_thr = imthera_data['threshold (mA)'].to_numpy()
imth_mrg_thr = imthera_data['mrggpu threshold'].to_numpy()

liv_thr = livanova_data['threshold (mA)'].to_numpy()
liv_mrg_thr = livanova_data['mrggpu threshold'].to_numpy()

plt.plot(np.arange(0, 21, 1), np.arange(0, 21, 1), 'lightgrey', linewidth=1)

plt.scatter(imth_mrg_thr, imth_thr, s=0.2, zorder=20, alpha=0.8, label='ImThera')
plt.scatter(liv_mrg_thr, liv_thr, s=0.2, zorder=10, alpha=0.8, label='Helical')

plt.axis('square')
plt.text(12, 9, f'$R^2$ = {r2:.3f}')
plt.ylabel('NEURON threshold (mA)')
plt.xlabel('S-MF threshold (mA)')
sns.despine(offset=6, trim=False)
plt.legend()

fig.savefig('./output/fig2a.svg', bbox_inches='tight')


# figure 2b

fig = plt.figure(dpi=200, figsize=(3, 3))

colors = sns.color_palette('colorblind')[:2]
styles = ['-', '--']
cuff = ['ImThera', 'Helical']
species = ['Human', 'Pig']

ax = sns.lineplot(x=r"diameter $(\mu m)$", y=r"absolute percentage error (%)",
                 hue='species', data=data,
                 palette="colorblind", hue_order = ['human', 'pig'], style='cuff',
                 errorbar='ci', legend=False)

ax1 = ax.twinx()
for ss, sty in enumerate(styles):
    ax1.plot(np.NaN, np.NaN, ls=styles[ss],
             label=cuff[ss], c='black')
ax1.get_yaxis().set_visible(False)
ax1.legend(loc='upper left', bbox_to_anchor=(1, 0.75))

ax2 = ax.twinx()
for ss, sty in enumerate(colors):
    ax2.plot(np.NaN, np.NaN,
             label=species[ss], c=sty)
ax2.get_yaxis().set_visible(False)
ax2.legend(loc='upper left', bbox_to_anchor=(1, 1))

ax.set_ylim(0, 3)
ax.set_xlabel("fiber diameter (μm)")
sns.despine(offset=6, trim=False)

fig.savefig('./output/fig2b.svg', bbox_inches='tight')


# figure 2c

fig, axes = plt.subplots(1, 2, dpi=200, figsize=(5.5, 3))

titles = ['ImThera', 'Helical']

for i, (ax, t) in enumerate(zip(axes.flatten(), titles)):
    ax.axhline(-5, linestyle='--', linewidth=1, color='black', alpha=0.6)
    ax.axhline(5, linestyle='--', linewidth=1, color='black', alpha=0.6)
    ax.axhline(-10, linestyle='--', linewidth=1, color='red', alpha=0.6)
    ax.axhline(10, linestyle='--', linewidth=1, color='red', alpha=0.6)
    ax.axhline(0, linestyle='--', linewidth=1, color='green', alpha=0.6)
    
    g = sns.violinplot(x=r"diameter $(\mu m)$", y=r"threshold error (\%)",
                       hue='species', data=data[(data['cuff']==t)],
                       palette="colorblind", showfliers=True,
                       whis=5, split=True, hue_order = ['human', 'pig'], cut=0, col='cuff', ax=ax,
                       legend=False)
    sns.despine(offset=6, trim=False)
    if i == 0:
        g.set_ylabel("threshold error (%)")
    else:
        g.set_ylabel("")
        
    if i == 1:
        ax.yaxis.set_ticklabels([])
        
    ax.xaxis.set_ticklabels([5.7, 6, 7, 8, 9, 10, 12, 14])
    g.set_xlabel("fiber diameter (μm)")

    ax.set_ylim(-15, 15)
    ax.get_legend().remove()

fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)
    
from matplotlib.lines import Line2D
from matplotlib.patches import Patch

cmap = sns.color_palette('colorblind')

custom_lines = [Patch(facecolor=cmap[0], edgecolor='black'),
                Patch(facecolor=cmap[1], edgecolor='black')]
plt.gca().legend(custom_lines, ['Human', 'Pig'], loc='upper center', bbox_to_anchor=(0.5, -0.22), ncols=2)

fig.savefig('./output/fig2c.svg', bbox_inches='tight')


# figure 2d

fig, axes = plt.subplots(1, 2, dpi=200, figsize=(5.5, 3))

titles = ['ImThera', 'Helical']

for i, (ax, t) in enumerate(zip(axes.flatten(), titles)):
    ax.axhline(-5, linestyle='--', linewidth=1, color='black', alpha=0.6)
    ax.axhline(5, linestyle='--', linewidth=1, color='black', alpha=0.6)
    ax.axhline(-10, linestyle='--', linewidth=1, color='red', alpha=0.6)
    ax.axhline(10, linestyle='--', linewidth=1, color='red', alpha=0.6)
    ax.axhline(0, linestyle='--', linewidth=1, color='green', alpha=0.6)
    
    g = sns.violinplot(x="stim", y=r"threshold error (\%)",
                       data=data[(data['cuff']==t)],
                       color=sns.color_palette('colorblind')[2], showfliers=True,
                       whis=5, cut=0, ax=ax, bw=0.3,
                       legend=False)
    sns.despine(offset=6, trim=False)
    if i == 0:
        g.set_ylabel("threshold error (%)")
    else:
        g.set_ylabel("")
        
    if i == 1:
        ax.yaxis.set_ticklabels([])
        
    g.set_xlabel("")

    ax.set_ylim(-15, 15)
    #ax.get_legend().remove()
    
for ax in axes.flatten():
    ax.set_xticks(ax.get_xticks(), ax.get_xticklabels(), rotation=45, ha='right')


fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)

fig.savefig('./output/fig2d.svg', bbox_inches='tight')


# figure 2e

peterson_data = pd.read_csv('./data/figure2/figure2e.csv')

fig = plt.figure(dpi=200, figsize=(3, 3))
sns.violinplot(x=r"diameter $(\mu m)$", 
               y="\% threshold error",
               data=peterson_data, 
               color=sns.color_palette('colorblind')[3], 
               cut=0)
sns.despine(offset=6, trim=False)

plt.xlabel('fiber diameter (μm)')
plt.ylabel('threshold error (%)')

plt.axhline(-5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(-10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)
plt.axhline(10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)

fig.savefig('./output/fig2e.svg', bbox_inches='tight')


# figure 2f

from matplotlib.lines import Line2D

df = pd.read_csv('./data/figure2/figure2f.csv')

cmap = sns.color_palette('colorblind')

fig, ax = plt.subplots(figsize=(4, 3), dpi=200)

g = sns.lineplot(data=df, x='size', y='time', hue='type', palette='colorblind',
                 ax=ax, hue_order=['NEURON 1 CPU', 'NEURON 350 CPU', 'Surrogate 1 GPU'])
sns.despine(offset=6, trim=False)

custom_lines = [Line2D([0], [0], color=cmap[0], lw=2),
                Line2D([0], [0], color=cmap[1], lw=2),
                Line2D([0], [0], color=cmap[2], lw=2)]

names = ['NEURON 1 CPU', 'NEURON 350 CPU', 'S-MF 1 GPU']
plt.gca().legend(custom_lines, names, loc='upper center', bbox_to_anchor=(0.5, -0.23), ncols=2)
plt.xlabel('# fibers')
plt.ylabel('time (s)')

g.set_xticks([1, 10, 100, 1000, 10000, 50000], minor=True)
g.set(xscale='log')
g.set(yscale='log')

fig.savefig('./output/fig2f.svg', bbox_inches='tight')