import matplotlib.pyplot as plt
import shapely
import numpy as np
import glob
import os
from natsort import natsorted
import seaborn as sns

import pandas as pd


from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
import matplotlib.font_manager as fm
import geopandas as gpd

fontprops = fm.FontProperties(size=8)

def get_inners(idx, samples_dir):
    all_inners = natsorted(glob.glob(f'./{samples_dir}/{idx}/slides/*/*/sectionwise2d/fascicles/*/inners/*.txt'))
    polys = []
    for inner in all_inners:
        coords = np.loadtxt(inner, skiprows=1)
        polys.append(shapely.Polygon(coords))
    return polys

def get_outers(idx, samples_dir):
    all_outers = natsorted(glob.glob(f'./{samples_dir}/{idx}/slides/*/*/sectionwise2d/fascicles/*/outer/*.txt'))
    polys = []
    for outer in all_outers:
        coords = np.loadtxt(outer, skiprows=1)
        polys.append(shapely.Polygon(coords))
    return polys

def get_nerve(idx, samples_dir):
    nerve = glob.glob(f'./{samples_dir}/{idx}/slides/*/*/sectionwise2d/nerve/*/*.txt')
    coords = np.loadtxt(nerve[0], skiprows=1)
    return shapely.Polygon(coords)

def plot_nerve(idx, samples_dir, save=False, figsize=(1.5, 1.5), numbered=False, highlight=None,
               fibers_xy=None, target_xy=None, active=None):
    inners = get_inners(idx, samples_dir)
    outers = get_outers(idx, samples_dir)
    nerve = get_nerve(idx, samples_dir)
    
    fig, ax = plt.subplots(dpi=200, figsize=figsize)
    
    for ix, i in enumerate(inners):
        c = (0.5, 0.5, 0.5, 0.3)
        if highlight is not None:
            if highlight[ix]:
                c = 'blue'
        # plt.plot(i.exterior.coords.xy[0], i.exterior.coords.xy[1], c=c, linewidth=0.4)
        p = gpd.GeoSeries(i)
        fc = 'none'
        if active is not None:
            if active[ix] and highlight[ix]:
                fc = (0, 1, 0, 0.5)
            elif active[ix] and not highlight[ix]:
                fc = (1, 0, 0, 0.5)
        p.plot(ax=ax, edgecolor=c, facecolor=fc, linewidth=0.5)
        if numbered:
            plt.text(i.centroid.xy[0][0], i.centroid.xy[1][0], ix)
        
    if fibers_xy is not None:
        for (x, y), t in zip(fibers_xy, active):
            c = 'red' if t else 'grey'
            plt.scatter(x, y, c=c, s=0.2)

    for i in outers:
        plt.plot(i.exterior.coords.xy[0], i.exterior.coords.xy[1], c=(0.5, 0.5, 0.5, 0.3), linewidth=0.4)
        
    plt.plot(nerve.exterior.coords.xy[0], nerve.exterior.coords.xy[1], c='black', linewidth=0.8)
    plt.axis('off')
    
    scalebar = AnchoredSizeBar(ax.transData,
                               500, '500 μm', 'lower right', 
                               pad=-0.5,
                               color='black',
                               frameon=False,
                               size_vertical=1,
                               fontproperties=fontprops)

    ax.add_artist(scalebar)
    
    r_nerve = (nerve.bounds[2] - nerve.bounds[0]) / 2
    return ax, r_nerve
    #if save:
    #    fig.savefig(f'./samples_{idx}.svg', bbox_inches='tight')


import json
import matplotlib

def degree_to_rad(degrees):
    return degrees * np.pi / 180

def plot_imthera(model_json, r_nerve, ax):
    with open(model_json, 'r') as f:
        model_data = json.load(f)
        
    pos_ang = degree_to_rad(model_data['cuff']['rotate']['pos_ang'])
    x_shift = model_data['cuff']['shift']['x']
    y_shift = model_data['cuff']['shift']['y']
    
    thk_medium_gap_internal_ITI = 10
    r_cuff_in_pre_ITI = 1498.6
    
    width = max(r_nerve+thk_medium_gap_internal_ITI,r_cuff_in_pre_ITI)
    diam_contact = 2000
    
    total_ang = (diam_contact / width)
    
    ang_cuffseam_contactcenter_ITC = degree_to_rad(53)*(r_cuff_in_pre_ITI/width)
    ang_contactcenter_contactcenter_ITC = degree_to_rad(51)*(r_cuff_in_pre_ITI/width)
    
    cp = sns.color_palette('colorblind')
    
    for i in range(6):
        start_ang = (ang_cuffseam_contactcenter_ITC+i*ang_contactcenter_contactcenter_ITC) - total_ang/2 
        end_ang = start_ang + total_ang
        
        if (i%2):
            scale = 1.04*1.05
        else:
            scale = 1.04*1

        arc_angles = np.linspace(start_ang+pos_ang, end_ang+pos_ang, 50)
        arc_xs = x_shift + scale * width * np.cos(arc_angles)
        arc_ys = y_shift + scale * width * np.sin(arc_angles)
        plt.plot(arc_xs, arc_ys, color = cp[i], lw = 3)
        

def plot_shrunk_imthera(model_json, r_nerve, ax):
    with open(model_json, 'r') as f:
        model_data = json.load(f)
        
    pos_ang = degree_to_rad(model_data['cuff']['rotate']['pos_ang'])
    x_shift = model_data['cuff']['shift']['x']
    y_shift = model_data['cuff']['shift']['y']
    
    thk_medium_gap_internal_ITI = 100
    r_cuff_in_pre_ITI = 1250
    
    width = max(r_nerve+thk_medium_gap_internal_ITI,r_cuff_in_pre_ITI)
    diam_contact = 2000
    
    total_ang = (diam_contact / width)
    
    ang_cuffseam_contactcenter_ITC = degree_to_rad(53)*(r_cuff_in_pre_ITI/width)
    ang_contactcenter_contactcenter_ITC = degree_to_rad(51)*(r_cuff_in_pre_ITI/width)
    
    cp = sns.color_palette('colorblind')
    
    for i in range(6):
        start_ang = (ang_cuffseam_contactcenter_ITC+i*ang_contactcenter_contactcenter_ITC) - total_ang/2 
        end_ang = start_ang + total_ang
        
        if (i%2):
            scale = 1.04*1.05
        else:
            scale = 1.04*1

        arc_angles = np.linspace(start_ang+pos_ang, end_ang+pos_ang, 50)
        arc_xs = x_shift + scale * width * np.cos(arc_angles)
        arc_ys = y_shift + scale * width * np.sin(arc_angles)
        plt.plot(arc_xs, arc_ys, color = cp[i], lw = 3)
        

def plot_livanova(model_json, r_nerve, ax):
    with open(model_json, 'r') as f:
        model_data = json.load(f)
        
    pos_ang = degree_to_rad(model_data['cuff']['rotate']['pos_ang'])
    x_shift = model_data['cuff']['shift']['x']
    y_shift = model_data['cuff']['shift']['y']


import os
os.makedirs('./output/cross_sections/', exist_ok=True)
samples = [2, 3, 4, 5, 7, 8, 110, 111, 112, 113, 114, 115]


df = pd.read_csv('./data/supplement/supplement_figure4.csv')


import ast

import warnings

warnings.filterwarnings(action='ignore', category=UserWarning)

for sample in samples:
    s = sample

    samples_dir = 'data/samples'
    s_df = df[df['id']==sample]
    target = ast.literal_eval(s_df[df['algorithm']=='target']['activations'].iloc[0])
    
    # biphasic first
    sub = s_df[s_df['waveform constraint']=='biphasic']
    
    # imthera
    imth = sub[sub['cuff']=='ImThera']
    algs = imth['algorithm'].unique()
    for alg in algs:
        active = ast.literal_eval(imth[imth['algorithm']==alg]['activations'].iloc[0])
        ax, r = plot_nerve(s, samples_dir, save=False, figsize=(3, 3), highlight=target, active=active)
        if sample < 100:
            plot_imthera(f'./{samples_dir}/{s}/models/0/model.json', r, plt.gca())
        else:
            plot_shrunk_imthera(f'./{samples_dir}/{s}/models/0/model.json', r, plt.gca())
        plt.gcf().savefig(f'./output/cross_sections/{s}_{alg}_biphasic.svg', bbox_inches='tight')
        plt.cla()
        plt.clf()
    
    # helical
    active = ast.literal_eval(sub[sub['cuff']=='Helical']['activations'].iloc[0])
    ax, r = plot_nerve(s, samples_dir, save=False, figsize=(3, 3), highlight=target, active=active)
    plt.gcf().savefig(f'./output/cross_sections/{s}_Helical_biphasic.svg', bbox_inches='tight')
    plt.cla()
    plt.clf()
    
    
    # arbitrary
    sub = s_df[s_df['waveform constraint']=='arbitrary']
    
    # imthera
    imth = sub[sub['cuff']=='ImThera']
    algs = imth['algorithm'].unique()
    for alg in algs:
        active = ast.literal_eval(imth[imth['algorithm']==alg]['activations'].iloc[0])
        ax, r = plot_nerve(s, samples_dir, save=False, figsize=(3, 3), highlight=target, active=active)
        if sample < 100:
            plot_imthera(f'./{samples_dir}/{s}/models/0/model.json', r, plt.gca())
        else:
            plot_shrunk_imthera(f'./{samples_dir}/{s}/models/0/model.json', r, plt.gca())
        plt.gcf().savefig(f'./output/cross_sections/{s}_{alg}_arbitrary.svg', bbox_inches='tight')
        plt.cla()
        plt.clf()


def timecourse(t, amp, pw, delay):
    p1 = amp * np.logical_and(
        t > delay, t <= delay + pw
    ).astype(float)
    p2 = -amp * np.logical_and(
        t > delay + pw, t <= delay + pw + pw
    ).astype(float)
    return p1 + p2


import os

os.makedirs('./output/optimized_waveforms/', exist_ok=True)

import seaborn as sns

df = pd.read_csv('./data/supplement/supplement_figure4_biphasic.csv')

cp = sns.color_palette('colorblind')
linecp = ['darkgrey', 'grey', 'black']
methods = ['DE + NEURON', 'DE + S-MF', 'GD + S-MF']
styles = ['-', '--', ':']

samples = [2, 3, 4, 5, 7, 8, 110, 111, 112, 113, 114, 115]

t = np.arange(0, 2, 0.005)
tc = timecourse(t, 1, 0.4, 0.4)

for sample in samples:
    fig, axes = plt.subplots(2, 3, sharex=True, sharey=True, dpi=200, figsize=(4.5, 3))
    s_df = df[df['id']==sample]
    for j, method in enumerate(methods):
        meth = s_df[s_df['algorithm']==method]
        contacts = meth['contact'].to_numpy()
        amplitudes = meth['amplitude'].to_numpy()
        for i, ax in enumerate(axes.flatten()):
            for spine in ax.spines.values():
                spine.set_color(cp[i])
                spine.set_linewidth(1.5)
            stim = amplitudes[i]*tc
            label = method
            ax.plot(t, stim, label=label, alpha=0.6, linewidth=1.5, color=linecp[j], linestyle=styles[j])

    labels = ['DE + NEURON', 'DE + S-MF', 'GD + S-MF']
    fig.legend(labels=labels, loc="lower center", ncol=2, bbox_to_anchor=(0.5, -0.2))

    # plt.ylim(-0.4, 0.4)

    fig.text(0.5, 0, 'time (ms)', ha='center')
    fig.text(-0.03, 0.5, 'current (mA)', va='center', rotation='vertical')
    fig.savefig(f'./output/optimized_waveforms/{sample}_biphasic.svg', bbox_inches='tight')
    plt.clf()


import seaborn as sns

df = pd.read_csv('./data/supplement/supplement_figure4_arbitrary.csv')


cp = sns.color_palette('colorblind')
linecp = ['darkgrey', 'black']
methods = ['DE + NEURON', 'GD + S-MF']

samples = [2, 3, 4, 5, 7, 8, 110, 111, 112, 113, 114, 115]
t = np.arange(0, 2, 0.005)


for sample in samples:
    fig, axes = plt.subplots(2, 3, sharex=True, sharey=True, dpi=200, figsize=(4.5, 3))
    s_df = df[df['id']==sample]
    for j, method in enumerate(methods):
        meth = s_df[s_df['algorithm']==method]
        xs = meth['i(t)']
        for i, (ax, x) in enumerate(zip(axes.flatten(), xs.items())):
            for spine in ax.spines.values():
                spine.set_color(cp[i])
                spine.set_linewidth(1.5)
            stim = ast.literal_eval(x[1])
            ax.plot(t, stim, label=method, alpha=0.6, linewidth=1.5, color=linecp[j])

    labels = ['DE + NEURON', 'GD + S-MF']
    fig.legend(labels=labels, loc="lower center", ncol=3, bbox_to_anchor=(0.5, -0.15))

    fig.text(0.5, 0, 'time (ms)', ha='center')
    fig.text(-0.03, 0.5, 'current (mA)', va='center', rotation='vertical')
    
    fig.savefig(f'./output/optimized_waveforms/{sample}_arbitrary.svg', bbox_inches='tight')
    plt.clf()